#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#  privacyIDEA
#  2016-09-05 Cornelius Kölbel <corneluis.koelbel@netknights.it>
#             Use new libnitrokey and cffi
#  2016-04-30 Cornelius Kölbel, <cornelius@privacyidea.org>
#             initial writeup
#
#  (c) 2016 Cornelius Kölbel, cornelius@privacyidea.org
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import binascii
import cffi
from enum import Enum


class DeviceErrorCode(Enum):
    STATUS_OK = 0
    NOT_PROGRAMMED = 3
    WRONG_PASSWORD = 4
    STATUS_NOT_AUTHORIZED = 5
    STATUS_AES_DEC_FAILED = 0xa

CDefs = """
void NK_set_debug(bool state);
int NK_login(const char *device_model);
int NK_login_auto();
int NK_logout();
const char * NK_status();
uint8_t NK_get_last_command_status();
int NK_lock_device();
int NK_user_authenticate(const char* user_password, const char* user_temporary_password);
int NK_first_authenticate(const char* admin_password, const char* admin_temporary_password);
int NK_factory_reset(const char* admin_password);
int NK_build_aes_key(const char* admin_password);
int NK_unlock_user_password(const char *admin_password, const char *new_user_password);
int NK_write_config(uint8_t numlock, uint8_t capslock, uint8_t scrolllock,bool enable_user_password, bool delete_user_password, const char *admin_temporary_password);
uint8_t* NK_read_config();
const char * NK_get_totp_slot_name(uint8_t slot_number);
const char * NK_get_hotp_slot_name(uint8_t slot_number);
int NK_erase_hotp_slot(uint8_t slot_number, const char *temporary_password);
int NK_erase_totp_slot(uint8_t slot_number, const char *temporary_password);
int NK_write_hotp_slot(uint8_t slot_number, const char *slot_name, const char *secret, uint8_t hotp_counter,bool use_8_digits, bool use_enter, bool use_tokenID, const char *token_ID,const char *temporary_password);
int NK_write_totp_slot(uint8_t slot_number, const char *slot_name, const char *secret, uint16_t time_window,bool use_8_digits, bool use_enter, bool use_tokenID, const char *token_ID,const char *temporary_password);
uint32_t NK_get_hotp_code(uint8_t slot_number);
uint32_t NK_get_hotp_code_PIN(uint8_t slot_number, const char* user_temporary_password);
uint32_t NK_get_totp_code(uint8_t slot_number, uint64_t challenge, uint64_t last_totp_time, uint8_t last_interval);
uint32_t NK_get_totp_code_PIN(uint8_t slot_number, uint64_t challenge,uint64_t last_totp_time, uint8_t last_interval, const char* user_temporary_password);
int NK_totp_set_time(uint64_t time);
int NK_totp_get_time();
int NK_change_admin_PIN(char *current_PIN, char *new_PIN);
int NK_change_user_PIN(char *current_PIN, char *new_PIN);
uint8_t NK_get_user_retry_count();
uint8_t NK_get_admin_retry_count();
int NK_enable_password_safe(const char *user_pin);
uint8_t * NK_get_password_safe_slot_status();
const char *NK_get_password_safe_slot_name(uint8_t slot_number);
const char *NK_get_password_safe_slot_login(uint8_t slot_number);
const char *NK_get_password_safe_slot_password(uint8_t slot_number);
int NK_write_password_safe_slot(uint8_t slot_number, const char *slot_name,const char *slot_login, const char *slot_password);
int NK_erase_password_safe_slot(uint8_t slot_number);
int NK_is_AES_supported(const char *user_password);
"""


class NitroKey(object):

    def __init__(self):
        self.ffi = cffi.FFI()
        for declaration in CDefs.split("\n"):
            self.ffi.cdef(declaration)
        self.nitrokey = self.ffi.dlopen("libnitrokey.so")
        self.session_password = None

    def admin_login(self, password):
        device_connected = self.nitrokey.NK_login_auto()
        if device_connected == 1:
            r = self.nitrokey.NK_set_debug(False)
            self.session_password = binascii.hexlify(os.urandom(8))
            r = self.nitrokey.NK_first_authenticate(password,
                                                    self.session_password)
            if r != 0:
                raise Exception("Failed to login. {0!s}".format(r))
        else:
            raise Exception("No Nitrokey connected. {0!s}".format(device_connected))

    def init_hotp(self, slot, slotname, otp_key=None,
                  digits=6):
        """
        Initilize the given slot with an HOTP key.

        If otp_key is not specified, then a random key is generated and
        returned.

        :param slot: The slot number, 15-17
        :type slot: int
        :param password: The card password
        :type password: basestring
        :param slotname: The new name of the slot
        :type slotname: basestring
        :param otp_key: The OTP secret key
        :type slotname: A hexstring
        :param digits: Number of OTP digits 6 or 8 allowed.
        :return: the otp key
        """
        if not self.session_password:
            raise Exception("You need to login as admin first!")
        if not 0 <= slot <= 2:
            raise Exception("Only HOTP slots between 0 and 2 are allowed.")
        use_8_digits = digits == 8
        if not otp_key:
            otp_key = os.urandom(10)
            otp_key_hex = binascii.hexlify(otp_key)
        else:
            otp_key_hex = otp_key

        r = self.nitrokey.NK_erase_hotp_slot(slot,
                                             self.session_password)
        if r != 0:
            #raise Exception("Error deleting OTP slot. {0!s}".format(r))
            print("Error deleting OTP slot. {0!s}".format(r))
        r = self.nitrokey.NK_write_hotp_slot(slot, slotname, otp_key, 0,
                                             use_8_digits, False, False, "",
                                             self.session_password)
        if r != 0:
            raise Exception("Error writing OTP slot. {0!s}".format(r))

        return otp_key_hex

    def logout(self):
        r = self.nitrokey.NK_logout()

    def status(self):
        """
        NK_status returns a cdata which can be transformed to this string:
        firmware_version:	7
        card_serial:	0000	a7 23 00 00
        general_config:	0000	01 00 ff 00 00
        numlock:	1
        capslock:	0
        scrolllock:	255
        enable_user_password:	0
        delete_user_password:	0
        :return:
        """
        r = self.nitrokey.NK_status()
        s = self.ffi.string(r)
        ret = {}
        for line in s.split("\n"):
            if ":" in line:
                key, val = line.split(":")
                if key:
                    ret[key] = val.strip()

        return ret


if __name__ == "__main__":
    Nk = NitroKey()
    Nk.admin_login("12345678")
    otp_key = Nk.init_hotp(2, "privadyidea")
    print otp_key
    print Nk.status()
    Nk.logout()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
'''
This module is used for enrolling yubikey

'''

try:
    import yubico
    import yubico.yubikey
    import yubico.yubikey_defs
    from yubico.yubikey import YubiKeyError
except ImportError as  e:
    print "python yubikey module not available."
    print "please get it from https://github.com/Yubico/python-yubico if you want to enroll yubikeys"
    print str(e)
    
from time import sleep
from usb import USBError
import sys
import re, os, binascii
try:
    from Crypto.Cipher import AES
    CRYPTO_AVAILABLE = True
except:
    CRYPTO_AVAILABLE = False
    print "No pycrypto available. You can not enroll yubikeys with static password."

MODE_YUBICO = 1
MODE_OATH = 2
MODE_STATIC = 3

hexHexChars = '0123456789abcdef'
modHexChars = 'cbdefghijklnrtuv'

hex2ModDict = dict(zip(hexHexChars, modHexChars))
mod2HexDict = dict(zip(modHexChars, hexHexChars))

def modhex_encode(s):
    return ''.join(
        [ hex2ModDict[c] for c in s.encode('hex') ]
    )

class YubiError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)


def create_static_password(key_hex):
    '''
    According to yubikey manual 5.5.5 the static-ticket is the same algorith with no moving factors.
    The msg_hex that is encoded with the aes key is '000000000000ffffffffffffffff0f2e'
    '''
    if not CRYPTO_AVAILABLE:
        raise Exception("No pycrypto available. You can not enroll Yubikey with static password!")
    
    msg_hex="000000000000ffffffffffffffff0f2e"
    msg_bin = binascii.unhexlify(msg_hex)
    aes = AES.new(binascii.unhexlify(key_hex), AES.MODE_ECB)
    password_bin = aes.encrypt(msg_bin)
    password = modhex_encode(password_bin)
    
    return password


def enrollYubikey(digits=6, APPEND_CR=True, debug=False, access_key=None,
                  new_access_key=None, slot=1,
                  mode=MODE_OATH,
                  fixed_string=None,
                  len_fixed_string=None,
                  prefix_serial=False,
                  challenge_response=False):
    """
    :param mode: Defines if the yubikey should be enrolled in OATH mode (1) or
        Yubico Mode (2)
    :type mode: integer
    
    :param fixed_string: A fixed string can be added in front of the output.
        If set to None, a random string will be generated
    :type fixed_string: string

    :param access_key: A hex string, that needs to be passed to programm the
        yubikey
    :param new_access_key: A hex string, that is set as the new access_key
    
    :param len_fixed_string: This specified the length of the random fixed
        string.
    :type len_fixed_string: integer
    
    :return: tuple of key, serial, fixed_string
    """
    YK = yubico.yubikey.find_key(debug=debug)
    firmware_version = YK.version()
    serial = "%08d" % YK.serial()
    # The fixed string to be returned
    ret_fixed_string = ""

    v1 = re.match('1.', firmware_version)
    v2 = re.match('2.0.', firmware_version)
    
    if v1 or v2:
        raise YubiError("Your Yubikey is too old. You need Firmware 2.1 or "
                        "above. You are running %s"
                        % firmware_version)

    Cfg = YK.init_config()

    # handle access_key and new_access_key
    if access_key:
        access_key = access_key if access_key[1:] == "h:" else "h:"+access_key
        Cfg.unlock_key(access_key)
    if new_access_key:
        # set new accesskey
        new_access_key = new_access_key if new_access_key[1:] == "h:" else \
            "h:" + new_access_key
        Cfg.access_key(new_access_key)

    # default fixed string length is 0, but 6 for MODE_YUBICO
    if len_fixed_string is None:
        if mode == MODE_YUBICO:
            len_fixed_string = 6
        else:
            len_fixed_string = 0

    if mode == MODE_YUBICO:
        key = binascii.hexlify(os.urandom(16))
        uid = os.urandom(yubico.yubikey_defs.UID_SIZE)
        if challenge_response:
            #Cfg.mode_challenge_response('h:' + key, type="OTP")
            raise YubiError("privacyIDEA only supports the OATH challenge "
                            "Response mode at the moment!")
        else:
            Cfg.mode_yubikey_otp(uid, 'h:' + key)

    elif mode == MODE_OATH:
        key = binascii.hexlify(os.urandom(20))
        if challenge_response:
            Cfg.mode_challenge_response('h:' + key, type="HMAC")
        else:
            try:
                # set hmac mode with key and 6 digits
                # Try if we got 0.0.5
                Cfg.mode_oath_hotp('h:' + key, digits=digits)
            except TypeError:
                # We seem to have 0.0.4
                Cfg.mode_oath_hotp('h:' + key, bytes=digits)

    elif mode == MODE_STATIC:
        key = binascii.hexlify(os.urandom(16))
        Cfg.aes_key('h:' + key)
        Cfg.config_flag('STATIC_TICKET', True)
        
    else:
        YubiError("Unknown OTP mode specified.")

    # Do the fixed string:
    if prefix_serial:
        ret_fixed_string = serial
    elif fixed_string:
        ret_fixed_string = fixed_string
    elif len_fixed_string:
        if mode == MODE_YUBICO:
            ret_fixed_string = binascii.unhexlify('ff') + os.urandom(
                len_fixed_string - 1)
        else:
            ret_fixed_string = os.urandom(len_fixed_string)
    Cfg.fixed_string(ret_fixed_string)
    ret_fixed_string = modhex_encode(ret_fixed_string)

    # set CR behind OTP value
    Cfg.ticket_flag('APPEND_CR', APPEND_CR)
    Cfg.extended_flag('SERIAL_API_VISIBLE', True)
    
    YK.write_config(Cfg, slot=slot)

    return key, serial, ret_fixed_string


def main():
    file_template = """<Tokens>
<Token serial="%s">
    <CaseModel>5</CaseModel>
    <Model>yubikey</Model>
    <ProductionDate>%s</ProductionDate>
    <ProductName>Yubikey 2.2</ProductName>
    <Applications>
        <Application ConnectorID="{a61c4073-2fc8-4170-99d1-9f5b70a2cec6}">
        <Seed>%s</Seed>
        <MovingFactor>1</MovingFactor>
        </Application>
    </Applications>
</Token>
</Tokens>"""

    import datetime
    from getopt import getopt, GetoptError

    OUTFILE = ""
    today = datetime.date.today().strftime("%m/%d/%Y")

    try:
        opts, args = getopt(sys.argv[1:], "o:",
                ['outfile='])

    except GetoptError:
        print "There is an error in your parameter syntax:"
        print "o, outfile=    the name of the output file"
        sys.exit(1)

    for opt, arg in opts:
        if opt in ('o', '--outfile'):
            print "setting output file : ", arg
            OUTFILE = arg

    #
    # example of usage
    #
    try:
        #otpkey, serial = enrollYubikey( debug= False ,
        #                                access_key = binascii.unhexlify('121212121212'),
        #                                unlock_key = binascii.unhexlify('121212121212'))
        otpkey, serial, fixed_string = enrollYubikey(debug=False)
        print "Success: serial: %s, otpkey: %s." % (serial, otpkey)
        #
        # Now we write to a file
        #
        if "" == OUTFILE:
            OUTFILE = "yubikey-%s.xml" % serial
        f = open(OUTFILE, 'w')
        f.write(file_template % ("YUBI%s" % serial, today, otpkey))
        f.close()

    except yubico.yubico_exception.YubicoError as  e:
        print "ERROR: %s" % str(e)
        sys.exit(1)
    except YubiError as e:
        print "Error: %s" % e.value


class YubikeyPlug(object):

    def __init__(self):
        self.last_serial = None

    def wait_for_new_yubikey(self, timeout=None):
        '''
        This functions waits for a new yubikey to be inserted
        '''
        found = False
        while 1:
            try:
                sleep(1)
                YK = yubico.yubikey.find_key()
                #firmware_version = YK.version()
                serial = "%08d" % YK.serial()

                if serial != self.last_serial:
                    self.last_serial = serial
                    print "\nFound Yubikey with serial %r\n" % serial
                    found = True
                    break;
            except USBError:
                sys.stdout.write('u')
                sys.stdout.flush()
            except YubiKeyError:
                sys.stdout.write('.')
                sys.stdout.flush()

        return found




if __name__ == "__main__":
    main()




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#  privacyIDEA
#  2015-03-16 Cornelius Kölbel, <cornelius@privacyidea.org>
#             initial writeup to fetch offline auth_items
#
#  (c) 2015 Cornelius Kölbel, cornelius@privacyidea.org
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Used to do offline authentication
"""
import base64
from os import urandom
import binascii
from hashlib import sha256
import sqlite3


def salted_hash_256(data, salt_length=16, salt=None):
    if not salt:
        b_ret = urandom(salt_length)
        salt = binascii.hexlify(b_ret)
    h = '{SSHA256}' + base64.b64encode(sha256(data + salt).digest() + salt)
    return h


def verify_salted_hash_256(data, h, salt_length=16):
    h_raw = h[len('{SSHA256}'):]
    h_bin = base64.b64decode(h_raw)
    salt = h_bin[-salt_length:]
    h_test = salted_hash_256(data, salt=salt)
    return h == h_test


def check_otp(user, otp, sqlfile="offlineotp.sqlite", window=10):
    """
    compare the given otp values with the next hashes of the user.

    DB entries older than the matching counter will be deleted from the
    database.

    :param user: The local user in the sql file
    :param otp: The otp value
    :param sqlfile: The sqlite file
    :return: True or False
    """
    conn = sqlite3.connect(sqlfile)
    c = conn.cursor()
    c.execute("SELECT * FROM authitems WHERE user='%s' ORDER by counter" % user)
    for x in range(0, window):
        r = c.fetchone()

        print r


def save_auth_item(sqlfile, authitem):
    """
    Save the given authitem to the sqlite file to be used later for offline
    authentication.

    There is only one table in it with the columns:

        username, counter, otp

    :param sqlfile: An SQLite file. If it does not exist, it will be generated.
    :type sqlfile: basestring
    :param authitem: A dictionary with all authitem information being:
    username, count, and a response dict with counter and otphash.

    :return:
    """
    conn = sqlite3.connect(sqlfile)
    c = conn.cursor()
    # Create the table if necessary
    try:
        c.execute("CREATE TABLE authitems "
                  "(counter int, user text, tokenowner text, otp text)")
    except:
        pass
    user = authitem.get("user")
    tokenowner = authitem.get("username")

    for counter, otphash in authitem.get("response").iteritems():
        # Insert the OTP hash
        c.execute("INSERT INTO authitems (counter, user, tokenowner, otp) "
                  "VALUES ('%s','%s','%s','%s')"
                  % (counter, user, tokenowner, otphash))

    # Save (commit) the changes
    conn.commit()

    # We can also close the connection if we are done with it.
    # Just be sure any changes have been committed or they will be lost.
    conn.close()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
__doc__ = """
This module is used for the communication of
the python based management clients "privacyidea"
"""

import logging.handlers
import pprint
import requests

from etokenng import (etng,
                      etngError)

import gettext

_ = gettext.gettext

TIMEOUT = 5
etng = False

file_opts = ['rf_file=']
ldap_opts = ['rl_uri=', 'rl_basedn=', 'rl_binddn=',
             'rl_bindpw=', 'rl_timeout=', 'rl_loginattr=',
             'rl_searchfilter=', 'rl_userfilter=',
             'rl_attrmap=']
ldap_opts_map = {'rl_uri': 'LDAPURI',
                 'rl_basedn': 'LDAPBASE',
                 'rl_binddn': 'BINDDN',
                 'rl_bindpw': 'BINDPW',
                 'rl_timeout': 'TIMEOUT',
                 'rl_searchfilter': 'LDAPSEARCHFILTER',
                 'rl_userfilter': 'LDAPFILTER',
                 'rl_attrmap': 'USERINFO',
                 'rl_loginattr': 'LOGINNAMEATTRIBUTE'
                 }


def showresult(rv):
    try:
        pp = pprint.PrettyPrinter(indent=4)
        print pp.pformat(rv['result'])
    except:
        print rv


class PrivacyIDEAClientError(Exception):
    """
    This class is used to throw client exceptions.
    """
    def __init__(self, eid=10, description="PrivacyIDEAClientError"):
        self.id = eid
        self.description = description
        
    def __str__(self):
        # here we lookup the error id - to translate
        return repr("ERR" + str(self.id) + ": " + self.description)


class response_obj(object):
    def __init__(self, status_code, json_object, text):
        self.status = status_code
        self.data = json_object
        self.text = text


class privacyideaclient(object):
    """
    This class is created to hold a connection to the privacyIDEA server

    It creates an authorization token that is sent in the HTTP header on each
    request.
    """

    def __init__(self, username, password, baseuri="http://localhost:5000",
                 no_ssl_check=False):
        """
        :param baseuri: The base of the server like http://localhost:5000
        :type baseuri: basestring
        """
        self.auth_token = None
        self.baseuri = baseuri
        self.log = logging.getLogger('privacyideaclient')
        self.verify_ssl = not no_ssl_check
        # Do the first server communication and retrieve the auth token
        self.set_credentials(username, password)

    def _send_response(self, r):
        if r.status_code >= 300:
            raise PrivacyIDEAClientError(eid=r.status_code,
                                          description=r.text)
        try:
            json_response = r.json
            if callable(json_response):
                # requests > 2.0
                json_response = json_response()
        except:
            json_response = None

        return response_obj(r.status_code, json_response, r.text)

    def set_credentials(self, username, password):
        """
        set an authtoken from the privacyidea server by providing username
        and password

        :param username: The username of the administrator or user
        :param password: The credential of the user
        :return: None
        """
        r = requests.post("%s/auth" % self.baseuri,
                          data={"username": username,
                                "password": password},
                          verify=self.verify_ssl)

        if r.status_code == requests.codes.ok:
            res = r.json
            if callable(res):
                # requests > 2.0
                res = res()
            result = res.get("result")
            if result.get("status") is True:
                self.auth_token = result.get("value", {}).get("token")
        else:
            raise Exception("Invalid Credentials: %s" % r.status_code)

    def get(self, uripath, param=None):
        r = requests.get("%s%s" % (self.baseuri, uripath),
                         headers={"Authorization": self.auth_token},
                         params=param,
                         verify=self.verify_ssl)
        return self._send_response(r)

    def post(self, uripath, param=None):
        r = requests.post("%s%s" % (self.baseuri, uripath),
                          headers={"Authorization": self.auth_token},
                          data=param,
                          verify=self.verify_ssl)
        return self._send_response(r)

    def delete(self, uripath):
        r = requests.delete("%s%s" % (self.baseuri, uripath),
                            headers={"Authorization": self.auth_token},
                            verify=self.verify_ssl)
        return self._send_response(r)

    def userlist(self, param):
        return self.get('/user/', param)

    def auditsearch(self, param):
        return self.get('/audit/', param)

    def inittoken(self, param):
        return self.post('/token/init', param)

    def listtoken(self, param):
        return self.get('/token/', param)

#    def getserialbyotp(self, param):
#        return self.connect('/admin/getSerialByOtp', param)

    def copytokenpin(self, param):
        return self.post('/token/copypin', param)

    def assigntoken(self, param):
        return self.post('/token/assign', param)

    def unassigntoken(self, param):
        return self.post('/token/unassign', param)

    def resetfailcounter(self, param):
        return self.post('/token/reset', param)

    def resynctoken(self, param):
        return self.post('/token/resync', param)

    def tokenrealm(self, serial, realms):
        return self.post('/token/realm/%s' % serial,
                         {'realms': realms})

    def set(self, param):
        """
        This function is used for many purposes like
            setmaxfail
            setsyncwindow
            setotplen
        This depends on the contents of the param dictionary.
        """
        return self.post('/token/set', param)

    def get_policy(self, param={}):
        return self.get('/policy/%s' % policy.get("name"), param)

    def setscpin(self, param):
        return self.post('/token/setpin', param)

    def disabletoken(self, param):
        param['enable'] = 'False'
        return self.post('/token/disable', param)

    def enabletoken(self, param):
        param['enable'] = 'True'
        return self.post('/token/enable', param)

    def deletetoken(self, serial):
        return self.delete('/token/%s'% serial)

    def getconfig(self, param):
        return self.get('/system/', param)

    def setconfig(self, param):
        return self.post('/system/setConfig', param)

    def deleteconfig(self, key):
        return self.delete('/system/%s' % key)

    def getrealms(self):
        return self.get('/realm/')

    def get_hsm(self, param={}):
        return self.get('/system/hsm', param)

    def set_hsm(self, param={}):
        return self.post('/system/hsm', param)

    def setrealm(self, realmname, param):
        return self.post('/realm/%s' % realmname, param)

    def deleterealm(self, realmname):
        return self.delete('/realm/%s' % realmname)

    def setdefaultrealm(self, realm):
        return self.post('/defaultrealm/%s' % realm)

    def deletedefaultrealm(self):
        return self.delete('/defaultrealm')

    def setresolver(self, resolver, param):
        response = None
        if 'type' not in param:
            raise PrivacyIDEAClientError(1202, _("When setting a resolver, "
                                                  "you need to specify "
                                                  "'type'."))
        if param["type"].lower() == "passwd":
            param["type"] = "passwdresolver"
            if 'filename' not in param:
                raise PrivacyIDEAClientError(1203, _("When setting a flat "
                                                     "file resolver, you need"
                                                     "to specify 'filename'."))

            response = self.post('/resolver/%s' % resolver, param)

        elif param['type'].lower() == 'ldap':
            raise NotImplementedError("TODO: Doing the Voodoo to set all "
                                      "these config keys")
        elif param['type'].lower() == 'sql':
            raise NotImplementedError("TODO: Doing the Voodoo to set all "
                                      "these config keys")
        else:
            raise PrivacyIDEAClientError(1204, _("Unknown resolver type"))

        return response

    def deleteresolver(self, resolvername):
        return self.delete("/resolver/%s" % resolvername)

    def getresolver(self, resolver=None):
        """
        Return a list of all resolvers.
        If a resolvername is specified, only this resolver is returned.
        :param resolver: resolvername
        :return:
        """
        if resolver:
            return self.get("/resolver/%s" % resolver)
        else:
            return self.get("/resolver/")


def dumpresult(status, data, tabformat=None):
    '''
    This function is used to print the Tokenlist in a nice viewable
    ascii table.
    '''
    if tabformat is None:
        tabformat = {'tabsize': [4, 16, 14, 20, 20, 4, 4, 4, 4],
                     'tabstr': ["%4s", "%16s", "%14s", "%20s", "%20s", "%4s", "%4s", "%4s", "%4s", "%4s"],
                        'tabdelim': '|',
                        'tabvisible': [0, 1, 2, 3, 4, 5, 6, 7, 8],
                        'tabhead': ['Id', 'Desc', 'S/N', 'User', 'Resolver',
                       'MaxFail', 'Active', 'FailCount', 'Window'],
                        'tabentry': ['id',
                        'description',
                        'serial',
                        'username',
                        'resolver',
                        'maxfail',
                        'active',
                        'failcount',
                        'sync_window']}

    tabsize = tabformat['tabsize']
    tabstr = tabformat['tabstr']
    tabdelim = tabformat['tabdelim']
    tabvisible = tabformat['tabvisible']
    tabhead = tabformat['tabhead']
    tabentry = tabformat['tabentry']

    # if not result['status']:
    if not status:
        print "The return status is false"
    else:

        head = tabentry

        # Set the default, if no tabformat or a wrong tabformat is passed
        if not len(tabvisible):
            for i in range(0, len(head)):
                tabvisible.append(i)
        if len(tabhead) < len(head):
            print "tabhead " + str(len(tabhead)) + " head " + str(len(head))
            for i in range(0, len(head)):
                tabhead.append('head')
        if len(tabsize) < len(head):
            for i in range(0, len(head)):
                tabsize.append(10)
        if len(tabstr) < len(head):
            for i in range(0, len(head)):
                tabstr.append("%10s")

        # value = result['value']
        # data = value['data']

        i = 0
        for t in tabhead:
            print tabstr[i] % str(t)[:tabsize[i]], tabdelim,
            i = i + 1
        print

        for token in data:
            i = 0
            for t in tabentry:
                # print tabstr[i] % str(token.get(t)).endcode('utf-8') [:tabsize[i]], tabdelim,
                # text=str(token.get(t)).encode('utf-8')
                text = token.get(t)
                if not type(token.get(t)) == unicode:
                    text = str(token.get(t))
                print tabstr[i] % text[:tabsize[i]], tabdelim,
                i = i + 1
            print



#!/usr/bin/python
# -*- coding: utf-8 -*-

" HMAC-OTP Software Token "

import os, sys, platform
import binascii
import hmac
from hashlib import sha1
import struct
from getopt import getopt, GetoptError


class HmacOtp:
    def __init__(self, key, counter=0, digits=6):
        self.key = key
        self.counter = counter
        self.digits = digits

    def hmac(self, key=None, counter=None):
        key = key or self.key
        counter = counter or self.counter
        digest = hmac.new(key, struct.pack(">Q", counter), sha1)
        return digest.digest()

    def truncate(self, digest):
        offset = ord(digest[-1:]) & 0x0f

        binary = (ord(digest[offset + 0]) & 0x7f) << 24
        binary |= (ord(digest[offset + 1]) & 0xff) << 16
        binary |= (ord(digest[offset + 2]) & 0xff) << 8
        binary |= (ord(digest[offset + 3]) & 0xff)

        return binary % (10 ** self.digits)

    def generate(self, key=None, counter=None):
        key = key or self.key
        counter = counter or self.counter
        otp = self.truncate(self.hmac(key, counter))
        self.counter = counter + 1
        return otp


def main():

    HEXKEY = "400edad7f3e8939c7ffa2d57d1bed94695bfd46c"
    TIMESTEP = 60
    OFFSET = 0

    def usage():
       print "o, offset=      tokenoffset in seconds"

    try:
        opts, args = getopt(sys.argv[1:], "o:",
                ['offset=', '--help'])

    except GetoptError:
        print "There is an error in your parameter syntax:"
        usage()
        sys.exit(1)

    for opt, arg in opts:
        if opt in ('o', '--offset'):
            print "setting offset : ", arg
            OFFSET = int(arg)




    from time import time
    counter = int((time() + OFFSET) / TIMESTEP + 0.5)

    key = binascii.a2b_hex(HEXKEY)
    otp = HmacOtp(key, counter=counter).generate()

    print "Your OTP with number %d is %06d." % (counter, otp)
    print "Happy Authenticating!"


if __name__ == '__main__':
    main()




#!/usr/bin/python
# -*- coding: utf-8 -*-
#    privacyIDEA is a fork of LinOTP
#    Dec 01, 2014 Cornelius Kölbel
#    License:  AGPLv3
#    contact:  http://www.privacyidea.org
#
#
#    LinOTP - the open source solution for two factor authentication
#    Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#
#    This file is part of LinOTP admin clients.
#
#    This program is free software: you can redistribute it and/or
#    modify it under the terms of the GNU Affero General Public
#    License, version 3, as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the
#               GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#    E-mail: linotp@lsexperts.de
#    Contact: www.linotp.org
#    Support: www.lsexperts.de
#
"""
This is the module for providing the functions
to enroll the Aladdin eToken NG OTP
"""

from ctypes import *
import binascii
import random
import platform
import logging
import logging.handlers

CKF_RW_SESSION = 0x00000002
CKF_SERIAL_SESSION = 0x00000004
CK_SAPI_OTP_HMAC_SHA1_DEC6 = 0x00000001
CKA_SAPI_OTP_MECHANISM = int(0x80001301)
CKA_SAPI_OTP_VALUE = int(0x80001304)
CKA_SAPI_OTP_NEXT_ALLOWED = int(0x80001306)
CKA_SAPI_OTP_DURATION = int(0x80001303)
CKR_MECHANISM_INVALID = int(0x00000070)

# AES
CKK_AES = int(0x0000001F)
CKA_CLASS = int(0x00000000)
CKO_DATA = int(0x00000000)
CKO_SECRET_KEY = int(0x00000004)
CKA_KEY_TYPE = int(0x00000100)
CKA_TOKEN = int(0x00000001)
CKA_LABEL = int(0x00000003)
CKA_ENCRYPT = int(0x00000104)
CKA_VALUE = int(0x00000011)
CKA_PRIVATE = int(0x00000002)

CKU_USER		 = 1
CKU_SO			 = 0


NULL = None


import locale
import gettext
locale.setlocale(locale.LC_ALL, '')
APP_NAME = "LinOTP2"
LOCALE_DIR = "locale"

_ = gettext.gettext
gettext.bindtextdomain(APP_NAME, LOCALE_DIR)
gettext.textdomain(APP_NAME)

#typedef CK_ULONG          CK_FLAGS;

#typedef struct CK_VERSION {
#  CK_BYTE       major;  /* integer portion of version number */
#  CK_BYTE       minor;  /* 1/100ths portion of version number */
#} CK_VERSION;
class CK_VERSION(Structure):
    _fields_ = [("major", c_byte),
                ("minor", c_byte),
                ]

class CK_TOKEN_INFO(Structure):
    _fields_ = [("label", c_wchar * 32),  # 0:31   Zeichen = 2byte
                ("manufacturerID", c_wchar * 32),  # 32:63
                ("model", c_wchar * 16),  # 64:79
                ("serialNumber", c_char * 16),  # 80:95
                ("flags", c_ulong),  # 96:97     4 byte
                ("ulMaxSessionCount", c_ulong),  # 98:99
                ("ulSessionCount", c_ulong),  # 100:101
                ("ulMaxRwSessionCount", c_ulong),  # 102:103
                ("ulRwSessionCount", c_ulong),  # 104:105
                ("ulMaxPinLen", c_ulong),  # 106:107
                ("ulMinPinLen", c_ulong),  # 108:109
                ("ulTotalPublicMemory", c_ulong),  # 110:111
                ("ulFreePublicMemory", c_ulong),  # 112:113
                ("ulTotalPrivateMemory", c_ulong),  # 114:115
                ("ulFreePrivateMemory", c_ulong),  # 116:117
                ("hardwareVersion", CK_VERSION),  # 118
                ("firmwareVersion", CK_VERSION),  # 119
                ("utcTime", c_char * 16),  # 120:135
                ]

class CK_ATTRIBUTE(Structure):
    _fields_ = [("type", c_ulong),
                ("pValue", c_void_p),
                ("ulValueLen", c_ulong),
                ]


class etngError(Exception):

    def __init__(self, id=10, description="etngError"):
        self.id = id
        self.description = description

    def __str__(self):
        ## here we lookup the error id - to translate
        return repr("ERR" + str(self.id) + ": " + self.description)


class etng(object):
  errormap = { 182:_('Session exists'),
                7:_('Bad argument'),
                19: _('Attribute value invalid'),
                162: _('invalid PIN length'),
                112: _('Mechanism invalid'),
                224: _('Token not present'),
                209: _('Template inconsistent'),
                208: _('Template incomplete'),
                163: _('PIN expired'),
                160: _('Unknown initializazion key')
                }

  def __init__(self, param):
    self.debug = False
    self.key = "123456789012345678901234"
    self.pw = ""
    self.npw = ""
    self.sopw = ""
    self.password = ""
    self.fpw = ""
    self.connectedTokens = []
    self.label = "newToken"
    self.displayDuration = 9
    self.randomUserPIN = True
    self.randomSOPIN = True

    # check params
    if 'label' in param:
        self.label = param['label']
    if 'debug' in param:
        self.debug = param['debug']
    if 'displayDuration' in param:
        self.displayDuration = param['displayDuration']
    if 'RetryCounter' not in param:
        self.retryCounter = 10
    else:
        self.retryCounter = param['RetryCounter']
    if 'sopin' in param:
        self.sopw = param['sopin']
    if 'userpin' in param:
        self.pw = param['userpin']
        self.password = param['userpin']
        self.npw = param['userpin']
    if 'randomUserPIN' in param:
        self.randomUserPIN = param['randomUserPIN'] == 'True'
    if 'randomSOPIN' in param:
        self.randomSOPIN = param['randomSOPIN'] == 'True'
    if 'logging' in param:
        self.logging = True
        self.handler = logging.handlers.RotatingFileHandler(
            param['logging']['LOG_FILENAME'],
            maxBytes=param['logging']['LOG_SIZE'],
            backupCount=param['logging']['LOG_COUNT'])
        self.formatter = logging.Formatter("[%(asctime)s][%(name)s][%(levelname)s]:%(message)s")
        self.handler.setFormatter(self.formatter)
        self.log = logging.getLogger("LinOTP etng")
        self.log.setLevel(param['logging']['LOG_LEVEL'])
        self.log.addHandler(self.handler)
    else:
        self.logging = False


    if self.logging: self.log.info("[init] setting label to %s and RetryCounter to %s" % (self.label, self.retryCounter))
    if self.debug: print "label: ", self.label
    if self.debug: print "RetryCounter: ", self.retryCounter

    self.tdata = { 'hmac':'', 'password':'', 'serial':'', 'error':'', 'sopassword':''}

    system = platform.system()
    if system == "Linux":
        self.etpkcs11 = CDLL("libeTPkcs11.so")
        self.etoken = self.etpkcs11
        self.etsapi = CDLL("libeTSapi.so")
    elif system == "Windows":
        self.etpkcs11 = CDLL("eTPkcs11")
        self.etoken = CDLL("etoken")
        self.etsapi = CDLL("eTSapi")
    else:
        raise etngError(2020, _("etng::__init__ - Unknown system platform (%s)") % system)
    self.hSession = c_ulong()

  def pkcs11error(self, rv):
    if rv in self.errormap:
        return self.errormap[rv]
    else:
        return rv

  def initpkcs11(self):
    self.etpkcs11.C_Initialize(0)
    self.connectedTokens = []
    # Get the number of connected Tokens
    prototype = CFUNCTYPE (c_int, c_int, POINTER(c_ulong), POINTER(c_ulong))
    paramflags = (1, "tokenPresent", 1), (2, "SlotID"), (2, "nSlots")
    getslotlist = prototype(("C_GetSlotList", self.etpkcs11) , paramflags)

    (SlotID, nSlots) = getslotlist()
    if self.logging: self.log.info("[initpkcs11] number of slots: %d, slotid: %d" % (nSlots, SlotID))
    if self.debug: print "Number of connected tokens: " , nSlots
    if self.debug: print "SlotID: " , SlotID

    if nSlots > 1:
        raise etngError(2020, _("etng::initpkcs11 - There are more than one tokens connected (%s)") % nSlots)

    if nSlots == 0:
        self.tdata['error'] = "No token connected"

  def createpasswd(self):
    pw = ""
    i = 0
    l = 10
    passwdchars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890!/()=?+-.,;:_<>*"
    while i < l:
        pw += passwdchars[ int(random.uniform(0, len(passwdchars) - 1))]
        i = i + 1
    return pw

  def inittoken(self):
    self.fpw = "1234567890"
    # we set a random user password
    if self.randomSOPIN:
        self.sopw = self.createpasswd()
    if self.randomUserPIN:
        self.password = self.createpasswd()
    self.npw = self.password

    # Start Initializing with SO PIN
    if self.logging: self.log.info ("[inittoken] token label: %s" % self.label)
    if self.logging: self.log.debug("[inittoken] sopw: %s" % self.sopw)
    if self.debug: print "label: ", self.label
    if self.debug: print "sopw:", self.sopw
    if self.debug: print "session:", self.hSession
    rv = self.etoken.ETC_InitTokenInit(0, self.sopw, len(self.sopw), 5, self.label, pointer(self.hSession))
    if rv:
        if self.logging: self.log.error("[inittoken] Failed to init token: %d" % rv)
        if self.debug: print "Failed to init token init: " , rv
        rv = self.pkcs11error(rv)
        raise etngError(2001, _("etng::inittoken - Failed to init token (%s)") % rv)
    else:
        if self.logging: self.log.info("[inittoken] init token succesful")
        if self.debug: print "session:", self.hSession
        if self.debug: print "init token init succesful"

    if self.logging: self.log.debug("[inittoken] npw: %s" % self.npw)
    if self.debug: print "npw  : ", self.npw

    rv = self.etoken.ETC_InitPIN (self.hSession, self.npw, len(self.npw), int(self.retryCounter), True)
    if rv:
        if self.logging: self.log.error("[inittoken] Failed User C_InitPin")
        if self.debug: print "Failed User C_InitPin"
        rv = self.pkcs11error(rv)
        raise etngError(2002, _("etng::inittoken - Failed to ETC_InitPIN (%s)") % rv)
    else:
        if self.logging: self.log.info("[inittoken] C_InitPIN successful")
        if self.debug: print "init token init User PIN succesful"

    rv = self.etoken.ETC_InitTokenFinal(self.hSession)
    if rv:
        if self.logging: self.log.error("[inittoken] Failed finalize InitTokenInit")
        if self.debug: print "Failed to finalize InitTokenInit"
        raise etngError(2003, _("etng::inittoken - Failed to ETC_InitTokenFinal (%s)") % rv)
    else:
        if self.logging: self.log.info("[inittoken] finalize InitTokenInit successful")
        if self.debug: print "init token init successfully finalized"

    # End of Initializing with SO PIN
    # This closes the initialization session

  def logintoken(self):
    # Open a session on fist token
    prototype = CFUNCTYPE (c_int, c_int, c_int, POINTER(c_ulong), POINTER(c_ulong), POINTER(c_ulong))
    paramflags = (1, "SlotID", 0), (1, "Flags", 6), (1, "App", NULL), (1, "Notify", NULL), (2, "SessionHandle")
    opensession = prototype(("C_OpenSession", self.etpkcs11), paramflags)
    self.hSession = opensession(SlotID=0)

    self.pw = self.password

    #print self.password
    #print self.pw
    #print len(self.pw)

    rv = self.etpkcs11.C_Login(self.hSession, CKU_USER, self.pw, len(self.pw))
    if rv:
        if self.logging: self.log.error("[logintoken] C_Login failed")
        if self.debug: print "Failed to login to token: " , rv
        raise etngError(2004, _("etng::logintoken - Failed to C_Login (%s)") % rv)
    else:
        if self.logging: self.log.info("[logintoken] C_Login successful")
        if self.debug: print "Login succesful"

  def deleteOTP(self):
    # Deleting existing OTP appliacion
    if self.logging: self.log.info("[deleteOTP] Deleting possible existing OTP application on the token")
    if self.debug: print "Deleting possible existing OTP application on the token"
    self.etsapi.SAPI_OTP_Destroy(self.hSession)

    # Creating random hmac key
    #Java Card OTP MinKeySize = 20
    #Java Card OTP MaxKeySize = 24
    #CardOS OTP MinKeySize = 20
    #CardOS OTP MaxKeySize = 32
    # TODO: zur Zeit 24 byte

    #self.key = "12345678901234567890"

    if self.logging: self.log.info("[deleteOTP] I will create a new HMAC key with this keysize: %d" % len(self.key))
    if self.debug: print "Sizeof key: ", len(self.key)

    rv = self.etpkcs11.C_GenerateRandom(self.hSession, self.key, c_ulong(len(self.key)))
    if rv:
        if self.logging: self.log.error("[deleteOTP] C_GenerateRandom failed %d" % rv)
        if self.debug: print "C_GenerateRandom failed:", rv
        raise etngError(2005, _("etng::deleteOTP - Failed to C_GenerateRandom (%s)") % rv)
    else:
        if self.logging: self.log.info("[deleteOTP] C_GenerateRandom successful")
        if self.logging: self.log.debug("[deleteOTP] New HMAC-Key: %s" % binascii.hexlify(self.key))
        if self.debug: print "created random ", len(self.key), " byte HMAC key:", binascii.hexlify(self.key)


  def createAESKey(self):
    if self.logging: self.log.info("[createOTP] About to create new OTP object ")
    if self.debug: print "Create new OTP object"

    #keyClass = c_ulong(CKO_SECRET_KEY)
    dClass = c_ulong(CKO_SECRET_KEY)
    keyType = c_ulong(CKK_AES)
    ck_true = c_ubyte(True)
    label = "my AES key"
    value = "1234567890123456"
    c_label = c_char_p(label)
    c_value = c_char_p(value)

    tC1 = CK_ATTRIBUTE(c_ulong(CKA_CLASS),
        cast(byref(dClass), c_void_p), sizeof(dClass))
    tC2 = CK_ATTRIBUTE(c_ulong(CKA_KEY_TYPE),
        cast(byref(keyType), c_void_p), sizeof(keyType))
    tC3 = CK_ATTRIBUTE(c_ulong(CKA_TOKEN),
        cast(byref(ck_true), c_void_p), sizeof(ck_true))
    tC4 = CK_ATTRIBUTE(c_ulong(CKA_LABEL),
        cast(c_label, c_void_p), len(label))
    tC5 = CK_ATTRIBUTE(c_ulong(CKA_ENCRYPT),
        cast(byref(ck_true), c_void_p), sizeof(ck_true))
    tC6 = CK_ATTRIBUTE(c_ulong(CKA_VALUE),
        cast(c_value, c_void_p), len(value))
    #tC7 = CK_ATTRIBUTE( c_ulong(CKA_PRIVATE),
    #    cast( byref( ck_true), c_void_p), sizeof( ck_true))

    arrayCK_ATTRIBUTES = CK_ATTRIBUTE * 6

    hObject = c_ulong()

    #tCreate = arrayCK_ATTRIBUTES( tC1, tC2, tC3, tC4, tC5, tC6)

    tCreate = (tC1, tC2, tC3, tC4, tC5, tC6)

    rv = self.etpkcs11.C_CreateObject(self.hSession, tCreate, len(tCreate), pointer(hObject))

    if rv:
        rv = self.pkcs11error(rv)
        if self.logging: self.log.error("[createAES] SAPI_OTP_Create failed: %d " % rv)
        if self.debug: print "Error creating AES object: ", rv
        raise etngError(2006, _("etng::createAES - Failed to etpkcs11.C_CreateObject (%s).") % rv)
    else:
        if self.logging: self.log.info("[createAES] C_CreateObject successful")
        if self.debug: print "AES object created successfully"


  def createOTP(self):
    # Creating new OTP object
    if self.logging: self.log.info("[createOTP] About to create new OTP object ")
    if self.debug: print "Create new OTP object"

    p_c_key = c_char_p(self.key)
    p_key = cast(p_c_key, c_void_p)

#    rv = self.mysapi.my_OTP_Create( self.hSession, p_key, c_ulong(20), self.displayDuration)
#CK_RV my_OTP_Create(CK_SESSION_HANDLE hSession, CK_BYTE_PTR RandomData, CK_ULONG ulRandomLen, CK_ULONG dduration)
#{
#  CK_ULONG mech = CK_SAPI_OTP_HMAC_SHA1_DEC6;
#  CK_BBOOL ck_false = FALSE;
#  CK_ATTRIBUTE tCreate[]= {
#	{CKA_SAPI_OTP_MECHANISM,    &mech,        sizeof(CK_ULONG)},
#  	{CKA_SAPI_OTP_VALUE,        RandomData,   ulRandomLen    },
#	{CKA_SAPI_OTP_DURATION,	    &dduration,		sizeof(CK_ULONG)},
#  CKA_SAPI_OTP_NEXT_ALLOWED, &ck_false,     sizeof(CK_BBOOL)},
# };
# return SAPI_OTP_Create(hSession, tCreate, sizeof(tCreate)/sizeof(CK_ATTRIBUTE));
#
#    print "new stuff"
    ck_mech = c_ulong(CK_SAPI_OTP_HMAC_SHA1_DEC6)
    ck_false = c_ubyte(False)
    ck_duration = c_ulong(int(self.displayDuration))
    tCreate1 = CK_ATTRIBUTE(c_ulong(CKA_SAPI_OTP_MECHANISM), cast(byref(ck_mech), c_void_p), sizeof(ck_mech))
    tCreate2 = CK_ATTRIBUTE(c_ulong(CKA_SAPI_OTP_VALUE), p_key, len(self.key))
    tCreate3 = CK_ATTRIBUTE(c_ulong(CKA_SAPI_OTP_NEXT_ALLOWED), cast(byref(ck_false), c_void_p), sizeof(ck_false))
    tCreate4 = CK_ATTRIBUTE(c_ulong(CKA_SAPI_OTP_DURATION), cast(byref(ck_duration), c_void_p), sizeof(ck_duration))
    arrayCK_ATTRIBUTES = CK_ATTRIBUTE * 4
    tCreate = arrayCK_ATTRIBUTES(tCreate1, tCreate2, tCreate3, tCreate4)
#    print len(tCreate)
#    print  "done 1"
    rv = self.etsapi.SAPI_OTP_Create(self.hSession, tCreate, len(tCreate))
#    print "done 2"

    if rv:
        rv = self.pkcs11error(rv)
        if self.logging: self.log.error("[createOTP] SAPI_OTP_Create failed: %d " % rv)
        if self.debug: print "Error creating OTP object: ", rv
        raise etngError(2006, _("etng::createOTP - Failed to etsapi.SAPI_OTP_Create (%s). Maybe the token was initialized previously without HMAC support?") % rv)
    else:
        if self.logging: self.log.info("[createOTP] SAPI_OTP_Create successful")
        if self.debug: print "OTP object created successfully"

  def finalize(self):
    # In fact we do not neet the CK_TOKEN_INFO Structure at the moment.
    tokeninfo = CK_TOKEN_INFO()
    tInfo = ''
    # Build up a 400 Byte mem alloc ;-)
    # The Structure is 400 bytes
    for i in range(40):
        tInfo += '0123456789'
    p_c_tokeninfo = c_char_p(tInfo)
    p_tokeninfo = cast(p_c_tokeninfo, c_void_p)
    rv = self.etpkcs11.C_GetTokenInfo(0, p_tokeninfo)
    if rv:
        if self.logging: self.log.error("[finalize] Error getting TokenInfo: %d" % rv)
        if self.debug: print "Error getting TokenInfo: ", rv
        raise etngError(2007, _("etng::Finalize - Failed to C_GetTokenInfo (%s)") % rv)
    else:
        if self.logging: self.log.info("[finalize] getting TokenInfo successful ")
        if self.debug: print "Got Token info successfully"

    # FIXME: We should make this more robust and use the structure
    _tiLabel = self.unpad(tInfo[0:31])
    _tiManufacturer = self.unpad(tInfo[32:63])
    _tiModel = self.unpad(tInfo[64:79])
    tiSerial = self.unpad(tInfo[80:95])
    serial = tiSerial

    rv = self.etpkcs11.C_Finalize(0)
    if rv:
        if self.logging: self.log.error("[finalize] Error finalizing Token %d" % rv)
        if self.debug: print "Error finalizing Token: ", rv
        raise etngError(2007, _("etng::Finalize - Failed to finalize token (%s)") % rv)
    else:
        if self.logging: self.log.info("[finalize] Token finalized successful ")
        if self.debug: print "Token finalized successfully"

    if self.logging: self.log.debug("[finalize] Token was created with the data HMAC-key (" +
                    binascii.hexlify(self.key) +
                    "), Serial (" + serial +
                    "), PIN (" + self.password +
                    "), SO PIN (" + self.sopw + "). YOU ASKED FOR IT! ;-)")
    if self.debug: print "Your data:"
    if self.debug: print " HMAC-key  : ", binascii.hexlify(self.key)
    if self.debug: print " Serial    : ", serial
    if self.debug: print " eToken PIN: ", self.password
    if self.debug: print " SO PIN    : ", self.sopw

    self.tdata = { 'hmac' : binascii.hexlify(self.key), 'serial':serial, 'userpin':self.password, 'sopin':self.sopw }
    return self.tdata

  def unpad(self, inValue):
    outValue = ""
    for i in range(len(inValue)):
        if inValue[i] == " ":
            break
        else:
            outValue += inValue[i]
    return outValue



##### plain function
def initetng(param={ 'label': 'newToken'}):
    debug = True
    if not 'label' in param:
        param['label'] = 'newToken'
    if not 'retrycounter' in param:
        param['RetryCounter'] = 15

    if 'debug' in param:
        debug = param['debug']


    enroller = etng(param)

    print _("initialize pkcs11 interface") if debug else '',
    enroller.initpkcs11()
    print _("initialize token")  if debug else '',
    enroller.inittoken()
    print _("login to token") if debug else '',
    enroller.logintoken()
    print _("delete old OTP application") if debug else '',
    enroller.deleteOTP()
    print _("create new OTP application") if debug else '',
    enroller.createOTP()
    print _("Creating AES key") if debug else '',
    #enroller.createAESKey()
    #print _("finalizing token") if debug else '',
    tdata = enroller.finalize()

    return tdata

if __name__ == "__main__":


    initetng({'userpin':'test123!', 'label':'eTokenNG', 'randomUserPIN':False, 'debug' : True })



import binascii
import os
KEY = "@ABCDEFGHIJKLMNO"
hotpKeyVersion = 0x03
counterFile = 0x42
kbFile = 0x0800
hidMappingFile = 0x0001

try:
    from daplug.conv import *
    from daplug.utils import *
    from daplug.keyset import *
    from daplug.keyboard import *
    from daplug import *
    
    secu01 = DaplugDongle.C_MAC
    secu03 = DaplugDongle.C_MAC + DaplugDongle.C_DEC
    defKeys = KeySet(0x01, binascii.hexlify(KEY))
    DAPLUG = True
except Exception as e:
    DAPLUG = False


def _daplug_missing_info():
    print("Can not find daplug python modules.")
    print("You will not be able to enroll daplug devices.")
    print("%s" % e)
    

def _delete_HOTP(dongle):
    # Clean HOTP key
    dongle.selectFile(0x3F00)
    try:
        dongle.deleteKeys([hotpKeyVersion])
    except DaplugException as e:
        print e
        pass
    try:
        dongle.selectFile(0xC010)
        dongle.deleteFileOrDir(counterFile)
    except DaplugException as e:
        print e
        pass
    try:
        dongle.setKeyboardAtBoot(False)
    except DaplugException:
        pass
    try:
        dongle.deleteFileOrDir(kbFile)
    except DaplugException:
        pass

    try:
        # Delete HID Mapping
        dongle.selectFile(0x3F00)
        dongle.deleteFileOrDir(hidMappingFile)
    except DaplugException:
        pass

    try:
        # Delete kbFIle
        dongle.selectFile(0x3F00)
        dongle.deleteFileOrDir(kbFile)
    except DaplugException:
        pass

    try:
        dongle.selectPath([DaplugDongle.MASTER_FILE, 0xC010])
        dongle.deleteFileOrDir(counterFile)
    except DaplugException:
        pass


def init_dongle(keyboard=False,
                mapping="05060708090a0b0c0d0e",
                otplen=6):
    if not DAPLUG:
        _daplug_missing_info()
        return (None, None)
    utils.DEBUG = False
    dongle = getFirstDongle()
    _serial = dongle.getSerial()
    serial = lst2hex(_serial)
    dongle.authenticate(defKeys, secu01)
    _delete_HOTP(dongle)
    '''Generate HOTP key and install it'''
    hotpkey = binascii.hexlify(os.urandom(20))
    keys = splitKey(hotpkey)
    HOTPKey = KeySet(hotpKeyVersion, keys[0], keys[1], keys[2])
    HOTPKey.setKeyAccess(0x0000 + 20)
    HOTPKey.setKeyUsage(KeySet.USAGE_HOTP)
    dongle.putKey(HOTPKey)
    
    # Create HID mapping file
    dongle.selectFile(0x3f00)
    dongle.createFile(hidMappingFile, 10)
    # HID mapping :   b c d e f g h i j k
    dongle.write(0, mapping)
    # Create a counter file
    dongle.createCounterFile(counterFile)
    # Create Keyboard file
    if keyboard:
        dongle.selectFile(0x3f00)
        dongle.createFile(kbFile, 64)
        
        # Prepare personalization file content
        kb = KeyBoard()
        kb.addSleep(0xFFFF)
        # kb.addSleep(0xFFFF)
        # kb.addSleep(0xFFFF)
        # kb.addSleep(0xFFFF)
        # 04 is to use HID mapping - 02 is for numeric (not very good)
        kb.addHotpCode(0x04, int(otplen),
                       hotpKeyVersion,
                       counterFile)
        kb.addReturn()
        kb.zeroPad(64)
        dongle.write(0, kb.content)
        
        # Set keyboard at boot
        dongle.useAsKeyboard()
        dongle.setKeyboardAtBoot(True)
        
        # Go HID if required
        if dongle.getMode() == "usb":
            dongle.usb2hid()
    else:
        if dongle.getMode() == "hid":
            dongle.hid2usb()
            
    dongle.deAuthenticate()
    
    return (serial, hotpkey)



# -*- coding: utf-8 -*-
from distutils.core import setup
import os
import sys

#VERSION="2.1dev4"
VERSION = "2.15.1"

# Taken from kennethreitz/requests/setup.py
package_directory = os.path.realpath(os.path.dirname(__file__))


def get_file_contents(file_path):
    """Get the context of the file using full path name."""
    content = ""
    try:
        full_path = os.path.join(package_directory, file_path)
        content = open(full_path, 'r').read()
    except:
        print >> sys.stderr, "### could not open file %r" % file_path
    return content


setup(name='privacyideaadm',
      version=VERSION,
      description='privacyIDEA admin Client',
      author='Cornelius Kölbel',
      author_email='cornelius@privacyidea.org',
      url='http://www.privacyidea.org',
      packages=['privacyideautils'],
      install_requires=["requests", "pysqlite"],
      scripts=['scripts/privacyidea',
               'scripts/privacyidea-luks-assign',
	           'scripts/privacyidea-authorizedkeys'],
#      install_requires=['usb'],
      data_files=[('share/man/man1', ["doc/_build/man/privacyidea.1"])],
      license='AGPLv3',
      long_description=get_file_contents('DESCRIPTION')
      )



#!/usr/bin/python
# -*- coding: utf-8 -*-

" HMAC-OTP Software Token "

import os, sys, platform
import binascii
import hmac
from hashlib import sha1
import struct

class HmacOtp:
    def __init__(self, key, counter=0, digits=6):
        self.key = key
        self.counter = counter
        self.digits = digits

    def hmac(self, key=None, counter=None):
        key = key or self.key
        counter = counter or self.counter
        digest = hmac.new(key, struct.pack(">Q", counter), sha1)
        return digest.digest()

    def truncate(self, digest):
        offset = ord(digest[-1:]) & 0x0f

        binary = (ord(digest[offset + 0]) & 0x7f) << 24
        binary |= (ord(digest[offset + 1]) & 0xff) << 16
        binary |= (ord(digest[offset + 2]) & 0xff) << 8
        binary |= (ord(digest[offset + 3]) & 0xff)

        return binary % (10 ** self.digits)

    def generate(self, key=None, counter=None):
        key = key or self.key
        counter = counter or self.counter
        otp = self.truncate(self.hmac(key, counter))
        self.counter = counter + 1
        return otp


def main():
    counter_file = ""

    system = platform.system()
    if system == "Linux":
	counter_file = os.path.join(os.getenv("HOME"), ".pytoken-counter")
    elif system == "Windows":
        counter_file = os.path.join(os.getenv("HOMEDRIVE"), os.getenv("HOMEPATH"), "\pytoken-counter")
    else:
        print "I do not know your operating system"
        sys.exit(1)


    if os.path.exists(counter_file):
        counter = int(file(counter_file).read().strip()) + 1
    else:
        counter = 0

    hexkey = "<put_your_hmac_here>"

    key = binascii.a2b_hex(hexkey)
    otp = HmacOtp(key, counter=counter).generate()

    print "Your OTP with number %d is %06d." % (counter, otp)
    print "Happy Authenticating!"

    file(counter_file, 'w').write(str(counter))

if __name__ == '__main__':
    main()

